package kraft.monitor.tools;

import java.util.*;
import java.util.Date;
import java.text.DateFormat;
import java.text.SimpleDateFormat;


/** Class used for debugging, calls to Debug.println will print out the given
  * string, if the debugflag has been set to true.
  *
  *@author Ted Francis
  */
//=================
public class Debug {
//=================

  /** Flags debugging messages on/off.
    * If this is set to <code>true</code> then subsequent calls to the 
    * <code>print</code> methods will be successful
    */
  public static boolean debugflag = false;


  /** Prints out the Vector given to it, if debugflag is true.
    * Relies on the Objects in the Vector having a 
    * <code>toString</code> method.
    */
  public static void printVector(Vector v){
    int c = 0;
    if(debugflag){
      Enumeration e = v.elements();
      while(e.hasMoreElements()){
	System.out.println("Elem "+c+": [" + e.nextElement() +"]");
	c++;
      }
    }
  }

  /** Equivalent to <code>System.out.print(String)</code>, if debugflag is 
    * true.
    */
  public static void print(String str){
    if(debugflag){
      System.out.print(str);
    }
  }

  /** Equivalent to <code>System.out.println(String)</code>, if debugflag is 
    * true.
    */
  public static void println(String str){
    if(debugflag){
      System.out.println(str);
    }
  }

  /** Equivalent to <code>System.out.println()</code>, if debugflag is true
    */
  public static void println() {
    if(debugflag){
      System.out.println();
    }
  }

  /** Print out the current time in the form hh:mm:ss.000 to System.out */
  public synchronized static void printTime(){
    if(debugflag){
      Date date = Calendar.getInstance().getTime();
      String time = Formatter.format(date);
      System.out.print(time);
    }
  }

  /** Returns the current time as a formatted string (hh:mm:ss.000) */
  public synchronized static String time(){
	Date date = Calendar.getInstance().getTime();
	return Formatter.format(date);
  }

}//end class Debug

/** Formats Date objects into a string of the form HH:mm:ss.SSS
  * Example:  13:05:34.980   (13:05 and 34.980 seconds).
  *
  *@author Ted Francis
  *@see SimpleDateFormat
  */
class Formatter {

  private static SimpleDateFormat dateFormatter = 
  (SimpleDateFormat)DateFormat.getTimeInstance();

  private static String datePattern = "HH:mm:ss.SSS";
  
  /** Formats the given date into the current pattern.
    * Default pattern is <code> "HH:mm:ss.SSS" </code>
    */
  //-------------------------------------
  public static String format(Date date){
  //-------------------------------------
    dateFormatter.applyPattern(datePattern);
    return dateFormatter.format(date);
  }

  /** Apply a new pattern for use with <code> format </code>.
    * See java.text.SimpleDateFormat for valid patterns.
    */
  //-----------------------------------------------
  public static void applyPattern(String pattern) {
  //-----------------------------------------------
    datePattern = pattern;
    dateFormatter.applyPattern(pattern);
  }
  
  /** Set a time zone other than the local time zone to be used 
    * with <code> format </code>
    */
  //---------------------------------------------
  public static void setTimeZone(TimeZone zone) {
  //---------------------------------------------
    dateFormatter.setTimeZone(zone);
  }

}//end class Formatter

