package kraft.monitor.tools;

import kraft.TermInString;

/** Provides functions to manipulate messages/parts of messages provided as
  * TermInStrings.
  */
public class KraftTerm {


  /** Get the value of a particular field in a list of form:
    * '.'(:(field,value), '.'(:(field2,value2),[]))
    *
    *@param list a list term in the format shown above
    *@param field the field name required, eg: sender
    *@returns the value in the :(field,value) pair matchin 'field' 
    *         or empty string if not found
    */
  // ----------------------------------------------------------------
  public static String getFieldValue(TermInString list,String field) 
    throws Exception{
  //-----------------------------------------------------------------
    String curr_field = "dummy";//to go into for loop at least once
    String[] pair;

    try{
      for(int i=1; !curr_field.equals(""); i++) {
	pair = getListPair(list, i);
	curr_field = pair[0];
	if(pair[0].equals(field))
	  return pair[1];
      }//end for
    }
    catch(Exception e){
      throw new Exception("Error parsing list of args for field "+field+": " + e);
    }
    return "";
  }//end getFieldValue



  /** Get the field and value pair of args at position 'pos' in a list of form:
    * '.'(:(field,value), '.'(:(field2,value2),[]))
    *
    *@param list a list term in the above syntax
    *@param pos  the position in the list of the pair. NB: first pos is 1
    *@returns an array of two Strings.  the field name at [0], the value at [1]
    *         the values of the array will be {"",""} if no pair at 
    *         the given position
    */
  public static String[] getListPair(TermInString list, int pos) 
    throws Exception{
    
   TermInString list_part, colon_part;
   String[]     rtn = {"",""};
   int          curr;

   if(pos<=0) throw new IndexOutOfBoundsException("Position in field list: "+pos);
     
   try{
     if(!list.functor.equals(".") || list.arity!=2) return rtn;
     
     list_part = list;
     
     for(curr = 1; /*end condition is below*/ ; curr++) {

       if(list_part==null || 
	  list_part.is_empty_list() ||
	  list_part.argument(1)==null || 
	  list_part.argument(2)==null)
	 {
	   return rtn; // ie. an empty array {"", ""}
	 }

       if(curr==pos) {
       	 colon_part = new TermInString(list_part.argument(1));
	 if(colon_part.functor.equals(":") && colon_part.arity==2) {
	   rtn[0] = colon_part.argument(1);
	   rtn[1] = colon_part.argument(2);
	   return rtn;
	 }
       }//end if(curr==pos)

       list_part  = new TermInString(list_part.argument(2));
       if( ! list_part.functor.equals("."))
	 return rtn;
     }//end while
   }
    catch(Exception e){
      throw new Exception("Error parsing list of fields for element "+pos+": "+
			  e.getClass().getName());
    }

  }//end getListPair



  /** Debugging method: breaks down a term into functor, arity and args
    * and prints them to System.err
    */
  public static void breakDownTerm(TermInString t) {
    
    System.err.println("Full term is: "+t.term);
    System.err.println();
    System.err.println("Functor = "+t.functor);
    System.err.println("Arity   = "+t.arity);
    for(int i =1;i<=t.arity;i++)
      System.err.println("Argument("+i+") = "+t.argument(i));
  }



  /** typing java kraft.monitor.tools.KraftTerm <a_term>
    * will cause the term to be broken down
    * Used for testing
    */
  public static void main(String argv[])
    {
      TermInString term;
      
      try {
	term=new TermInString(argv[0]);
	breakDownTerm(term);
      } catch (Exception e) {
	System.err.println("Goddamnit:"+e.getMessage());
      }
    }//end main


}//end class KraftTerm
